
/*
 * newdefaults.c
 *
 * - A new way to store defaults internal to the client
 *   The old method was becoming quite a pain.  Calls to get
 *   the values were scattered all over the code.
 *   Also there was a lot of duplication of text descriptions
 *   in defwin.c, option.c as well as getdefaults.c
 *   This method will save space, and make it easier to read.
 *
 * 9/27/94  Steve Sheldon
 *
 */
#include "copyright.h"

#include <stdio.h>
#include <sys/types.h>
#include "Wlib.h"
#include "defs.h"
#include "struct.h"
#include "data.h"

/* The following is where the new style default's are stored SRS */

/* THESE MUST BE IN THE SAME ORDER AS THE ENUM CONSTANTS DEFINED
   IN defs.h.  MUST MUST MUST!!!! */
   
struct IntegerDefault IntVal[] =        /* baseUdpLocalPort */
{
  {
     "BaseUDPLocalPort", "Base UDP Local Port", 0,
     {0,0,0},
     {
       {
         0, ""
       }
       ,
       {
         0, NULL
       }
       ,
     }
     ,
  }
  ,
  {					/* phas_msg */
     "phasmsg", "Show phaser messages in windows", 0,
     {0,4,1},
     {
       {
         0, "Don't show phaser messages"
       }
       ,
       {
         1, "Show phaser messages on team window"
       }
       ,
       {
         2, "Show phaser messages on indiv window"
       }
       ,
       {
         3, "Show phaser messages on kill window"
       }
       ,
       {
         4, "Show phaser messages on total window"
       }
       ,
       {
         0, NULL
       }
     }
  } 
  ,
  {					/* showlocal */
     "showlocal", "Local planet bitmaps", 1,
     {0,2,1},
     {
       {
         0, "Show owner on local map"
       }
       ,
       {
         1, "Show resources on local map"
       }
       ,
       {
         2, "Show nothing on local map"
       }
       ,
       {
         0, NULL
       }
     }
  } 
  ,
  { 					/* showgalactic */
     "showGalactic", "Galactic planet bitmaps", 1,
     {0,2,1},
     {
       {
         0, "Show owner on galactic map"
       }
       ,
       {
         1, "Show resources on galactic map"
       }
       ,
       {
         2, "Show nothing on galactic map"
       }
       ,
       {
         0, NULL
       }
     }
  } 
  ,
  {					/* port */
     "port", "Default Server Port", 2592,
     {0,0,0},
     {
       {
         0, ""
       }
       ,
       {
         0, NULL
       }
     }
  }
  ,
  {					/* showLock */
     "showLock", "Lock display for planets/players", 3,
     {0,3,1},
     {
       {
         0, "Don't show lock"
       }
       ,
       {
         1, "Show lock on galactic only"
       }
       ,
       {
         2, "Show lock on tactical only"
       }
       ,
       {
         3, "Show lock on both"
       }
       ,
       {
         0, NULL
       }
     }
  }
  ,
  {					/* udpDebug */
     "udpDebug", "UDP Debugging Info", 0,
     {0,2,1},
     {
       {
         0, "OFF"
       }
       ,
       {
         1, "ON (connect msgs only)"
       }
       ,
       {
         2, "ON (verbose output)"
       }
       ,
       {
         0, NULL
       }
     }
  }
  ,
  {					/* udpClientSend */
     "udpClientSend", "UDP Send mode", 1,
     {0,3,1},
     {
       {
         0, "TCP only"
       }
       ,
       {
         1, "simple UDP"
       }
       ,
       {
         2, "enforced UDP (state only)"
       }
       ,
       {
         3, "enforced UDP (state & weapon)"
       }
       ,
       {
         0, NULL
       }
     }
  }
  ,
  {					/* udpClientRecv */
     "udpClientReceive", "UDP receive mode", 1,
     {0,2,1},
     {
       {
         0, "TCP only"
       }
       ,
       {
         1, "simple UDP"
       }
       ,
       {
         2, "fat UDP"
       }
       ,
       {
         0, NULL
       }
       ,
     }
     ,
  }
  ,
  {					/* enemyPhasers */
     "enemyPhasers", "Width of enemy phasers", 1,
     {1,60,1},
     {
       {
         0, ""
       }
       ,
       {
         0, NULL
       }
       ,
     }
     ,
  }
  ,
  {					/* tclock */
     "clock", "Clock display", 2,
     {0,2,1},
     {
       {
         0, "No clock"
       }
       ,
       {
         1, "Clock format hours:minutes"
       }
       ,
       {
         2, "Clock format hours:minutes:seconds"
       }
       ,
       {
         0, NULL
       }
       ,
     }
     ,
  }
  ,
  {					/* updatespeed */
     "updateSpeed", "Updates per second", 5,
     {1,10,1},
     {
       {
         0, ""
       }
       ,
       {
         0, NULL
       }
       ,
     }
     ,
  }
  ,
  {					/* mapmode */
     "GalacticFreq", "How fast galactic updates", 5,
     {0,2,1},
     {
       {
         0, "Don't update galactic map"
       }
       ,
       {
         1, "Update galactic map rarely"
       }
       ,
       {
         2, "Update galactic map frequently"
       }
       ,
       {
         0, NULL
       }
       ,
     }
     ,
  }
  ,

  #ifdef NETSTAT
  {				         /* netstatfreq */
     "netStatFreq", "Frequency of updates to calc lag", 5,
     {1,9999,1},
     {
       {
         0, ""
       }
       ,
       {
         0, NULL
       }
       ,
     }
     ,
  }
  ,
  #endif /* NETSTAT */
  #ifdef BEEPLITE
  {				/* beep_lite_cycle_time_planet */
     "PlanetCycleTime", "No. of updates to highlight planet", 10,
     {0,0,0};
     {
       {
         0, ""
       }
       ,
       {
         0, NULL
       }
       ,
     }
     ,
  }
  ,
  {				/* beep_lite_cycle_time_player */
     "PlayertCycleTime", "No. of updates to highlight player", 10,
     {0,0,0},
     {
       {
         0, ""
       }
       ,
       {
         0, NULL
       }
       ,
     }
     ,
  }
  ,
  #endif /* BEEPLITE */
  
  #ifdef MOTION_MOUSE
  {				/* user_motion_thresh */
     "Motionthresh", "Motion threshold for continuous mouse", 16,
     {0,0,0},
     {
       {
         0, ""
       }
       ,
       {
         0, NULL
       }
       ,
     }
     ,
  }
  ,
  #endif /* MOTION_MOUSE */
  
  #ifdef KEEP_INFO
  {				/* keepInfo */
     "KeepInfo", "Updates to keep info windows", 15,
     {0,0,0},
     {
       {
         0, ""
       }
       ,
       {
         0, NULL
       }
       ,
     }
     ,
  }
  ,
  #endif /* KEEP_INFO */
  
  #ifdef ZZ_BITMAPS
  {				/* myPlanetBitmap */
     "whichNewPlanetBitmaps", "MOO planet bitmaps", 0,
     {0,2,1},
     {
       {
         0, "Use normal bitmaps"
       }
       ,
       {
         1, "Use rabbit ear bitmaps"
       }
       ,
       {
         2, "Use ZZ minimal bitmaps"
       }
       ,
       {
         0, NULL
       }
       ,
     }
     ,
  }
  #endif /* ZZ_BITMAPS */
};

struct BooleanDefault BoolVal[] =
{
  {				/* namemode */
    "showPlanetNames", "Show planet names", TRUE
  }
  ,
  {				/* showShields */
     "showShields", "Show ship shields", TRUE
  }
  , 
  {				/* showStats */
     "showStats", "Show statistics window", TRUE
  }
  ,
  {				/* keeppeace */
     "keepPeace", "Stay peaceful when reborn", FALSE
  }
  ,
  {		/* phaserWindow */
     "phaserwindow", "Show phaser log Window", FALSE
  }
  , 
  {	/* reportKills */
     "reportKills", "Report kill messages", TRUE
  }
  , 
  {	/* showTractorPressor */
     "showTractorPressor", "show tractor/pressor", TRUE
  }
  , 
  {	/* fillTriangle */
     "fillTriangle", "Fill lock triangle", FALSE
  }
  , 
  {	/* continueTractor */
     "continueTractor", "Show tractors after lock", TRUE
  }
  , 
  {	/* extraBorder */
     "extraAlertBorder", "Alert on extra border(s)", TRUE
  }
  , 
  #ifdef NETSTAT
  {		/* netstat */
     "netstat", "show LagMeter", FALSE
  }
  , 
  #endif /* NETSTAT */
  {		/* warp */
     "warp", "Use message warp", TRUE
  }
  , 
  {		/* tryUdp */
     "tryudp", "Enable UDP packets", TRUE
  }
  , 
  {	/* udpSequenceChk */
     "udpSequenceCheck", "Check sequence of UDP packets", TRUE
  }
  , 
  {	/* galacticFrequent */
     "galacticFrequent", "Update galactic map frequently", TRUE
  }
  , 
  #ifdef LOGMESG
  {		/* logMess */
     "logMessage", "Log messages", FALSE
  }
  , 
  #endif
  #ifdef VSHIELD_BITMAPS
  {	/* VShieldBitmaps */
     "varyShields", "Change shields to reflect damage", FALSE
  }
  , 
  #endif
  {		/* use_msgw */
     "useMsgw", "Display last message in warning window.", TRUE
  }
  , 
  #ifdef RECORD
  {		/* recordGame */
     "recordGame", "Record the game to a file", FALSE
  }
  , 
  #endif
  #ifdef SHORT_PACKETS
  {		/* tryShort */
     "tryShort", "Enable short packets", TRUE
  }
  ,
  #endif
  #ifdef EM 
  {	/* sortPlayers */
     "sortPlayers", "Sort player listings by team", FALSE
  }
  , 
  #endif
  #ifdef PLIST
  {		/* newPlist */
     "newPlist", "New player list", TRUE
  }
  , 
  #endif
  {	/* new_messages */
     "newMesgFlags", "Use new message flags", TRUE
  }
  , 
  {	/* abbr_kmesg */
     "shortKillMesg", "Align kill messages", FALSE
  }
  , 
  #ifdef FEATURE
  #ifdef BEEPLITE
  {		/* DefLite */
     "defLite", "Highlight/use default BEEPlite", FALSE
  }
  , 
  {		/* UseLite */
     "useLite", "Use RCD highlighting", FALSE
  }
  , 
  #endif /* FEATURE */
  #endif /* BEEPLITE */
  #ifdef DASHBOARD
  {		/* dashboard */
     "dashboard", "Use new dashboard", FALSE
  }
  , 
  #endif /* DASHBOARD */
  #ifdef MOTION_MOUSE
  {	/* motion_mouse */
     "continuousMouse", "Use continuous mouse tracking", FALSE
  }
  , 
  #endif /* MOTION_MOUSE */
  #ifdef SHIFTED_MOUSE
  {	/* shiftedMouse */
     "shiftedMouse", "Use shifted mouse buttons", FALSE
  }
  , 
  #endif /* SHIFTED MOUSE */
  #ifdef TNG_FED_BITMAPS
  {	/* useTNGBitmaps */
     "useTNGBitmaps", "Use TNG style bitmaps", FALSE
  }
  , 
  #endif /* TNG_FED_BITMAPS */
  #ifdef VARY_HULL
  {		/* vary_hull */
     "warnHull", "warn hull", FALSE
  }
  , 
  #endif /* VARY_HULL */
  { 	/* plshowstatus */
     "showPlayerStatus", "show player status", TRUE
  }
  , 
  #ifdef SHOW_IND
  {		/* showIND */
     "showIND", "Show independant planets with X", TRUE
  }
  , 
  #endif /* SHOW_IND */
  #ifdef ROMVLVS_BITMAPS
  {		/* ROMVLVS */
     "ROMVLVS", "Use ROMVLVS style bitmaps", FALSE
  }, 
  #endif /* ROMVLVS BITMAPS */
  #ifdef USE_TR_PORTS
  {		/* USETRPORTS */
     "useTRports", "Use traceroute ports", FALSE
  }
  , 
  #endif /* USE_TR_PORTS */
  #ifdef MAP_NAMES
  {	/* draw_map_names */
     "showMapPlanetNames", "Show map planet names", TRUE
  }
  , 
  {	/* show_owner_map */
     "showOwnerOnMap", "Show owner on map", FALSE
  }
  , 
  #endif /* MAP_NAMES */
  #ifdef SHOW_MY_SPEED
  {	/* showMySpeed */
     "showMySpeed", "Show speed on tactical", FALSE
  }
  , 
  #endif /* SHOW_MY_SPEED */
  #ifdef JUBILEE_PHASERS
  {	/* jubilee_phasers */
     "colorfulPhasers", "Use colorful phasers", FALSE
  }
  , 
  #endif /* JUBILEE_PHASERS */
  #ifdef SHOW_FUEL_ON_LOCAL
  {	/* showFuelOnLocal */
     "showFuelOnLocal", "Show fuel on tactical", FALSE
  }
  , 
  #endif /* SHOW FUEL ON LOCAL */
  {	/* askforUpdate */
     "askForUpdate", "Request update on enter", FALSE
  }
  , 
  {	/* zeroArgUsage */
     "zeroArgUsage", "Print usage instead of looking for default server", FALSE
  }
  , 
  #ifdef RSA
  {		/* RSA_Client */
     "useRSA", "Use RSA verification", TRUE
  }
  , 
  #endif
  #ifdef FOR_MORONS
  {		/* For_Morons */
     "forMorons", "Use Special Moronic mode", FALSE
  }
  , 
  #endif
  #ifdef MOO
  {		/* showMotdOnQ */
     "waitMOTD", "Show Motd on the wait queue", TRUE
  }
  , 
  #endif /* MOO */
  #ifdef PING
  {		/* DontPing */
     "dontPing", "Don't use ping packets", FALSE
  }
  , 
  #endif
  {		/* forceMono */
     "forceMono", "Force Mono display", TRUE
  }

};

#ifdef nodef
   /* We're not going to try to keep track of string variables */
struct StringDefault StrVal[]=
{
  {		/* name */
    "name", "Default player name", ""
  }
  ,
  {		/* password */
    "password", "Default player password", ""
  }
  ,
  {		/* keymap */
    "keymap", "Keyboard map", ""
  }
  ,
  {		/* ckeymap */
    "ckeymap", "Control keyboard map", ""
  }
  ,
  {		/* playerlist */
    "Playerlist", "What to show on player list", ""
  }
  ,
  {		/* buttonmap */
    "bmap", "Mouse button map", ""
  }
  ,
  {		/* motd.savefile */
    "motd.savefile", "Where to save MOTD", ""
  }
  ,
#ifdef RECORD
  {		/* recordFile */
    "recordFile", "Name of file to record game in", ""
  }
  ,
#endif
  {		/* macroKey */
    "macroKey", "Name of key to use for macro escape", ""
  }
  ,
  {		/* cloakChars */
    "cloakChars", "Chars to draw cloakers with on map", ""
  }
  ,
  {		/* logFile */
    "logFile", "Name of file to log messages to", ""
  }
  ,
  {		/* serverName */
    "Server", "Default server name", ""
  }
};

InitStringDefaults() {

strcpy(StrVal[STR_MACROKEY].value, "X");
strcpy(StrVal[STR_CLOAKCHARS].value, "??");
strcpy(StrVal[STR_SERVER].value , "wormhole.ecst.csuchico.edu");
strcpy(StrVal[STR_PLAYERLIST].value , "nTR N  K lrSd");

};
#endif

#ifdef RECORD
extern char *recordFileName;    /* from main.c */
#endif

#ifdef LOGMESG
extern char *logFileName;       /* from main.c */
#endif


newreadDefaults()
{
  int i;
  printf ("Reading default values ...\n");

#ifdef nodef
  InitStringDefaults();
#endif
  
#define DEBUG

/* Get Integer defaults */

#ifdef DEBUG
  printf ("Reading Int default values ...\n");
#endif  
  for(i = 0; i < INT_MAX_DEFAULTS; i++)
  {
#ifdef DEBUG
  printf("%s = %d \n", IntVal[i].name, intDefault(IntVal[i].name, IntVal[i].value));
#endif

    IntVal[i].value = intDefault(IntVal[i].name, IntVal[i].value);

    if ( IntVal[i].range.increment ) {
      if (IntVal[i].value < IntVal[i].range.min_value)
        IntVal[i].value = IntVal[i].range.min_value;
      if (IntVal[i].value > IntVal[i].range.max_value)
        IntVal[i].value = IntVal[i].range.max_value;
    }
  }
  
  for(i = 0; i < BOOL_MAX_DEFAULTS; i++)
  {
#ifdef DEBUG
  printf("%s = %d \n", BoolVal[i].name, booleanDefault(BoolVal[i].name, IntVal[i].value));
#endif

    BoolVal[i].value = booleanDefault(BoolVal[i].name, BoolVal[i].value);
  };

#ifdef nodef      
  for(i = 0; i < STR_MAX_DEFAULTS; i++)
  {
#ifdef DEBUG
  printf("%s = %d \n", StrVal[i].name, getdefault(StrVal[i].name));
#endif

   if (getdefault(StrVal[i].name))
     strcpy(StrVal[i].value,getdefault(StrVal[i].name));

  }
#endif

  /* Read keymap */
  initkeymap ();


#ifdef RECORD
  /* Get the record filename and message log file from .xtrekrc */
  
  if (recordFileName == NULL)
    {
      recordFileName = getdefault ("recordFile");
      if (recordFileName != NULL)
	{
	  recordFile = fopen (recordFileName, "wb");
	  if (recordFile == NULL)
	    {
	      perror (recordFileName);
	      EXIT (1);
	    }
	  recordGame = 1;	/* We want to record the game */
	}
      else
	recordGame = 0;
    }
#endif

#ifdef BEEPLITE
    if (BoolVal[BOOL_DEFLITE])
      litedefaults ();
#endif

#ifdef FEATURE
  {
    char *macrokey_s = getdefault ("macroKey");
    
    if (macrokey_s)
      {
	if (strcmp (macrokey_s, "TAB") == 0)
	  macrokey = 9;
	else if (strcmp (macrokey_s, "ESC") == 0)
	  macrokey = 27;
	else
	  macrokey = (int) macrokey_s[0];
      }
    }
#endif

#ifdef CLOAK_CHARS
  pek = getdefault ("cloakChars");
  if (pek != (char *) NULL)
    strncpy (cloakChars, pek, 2);
#endif

#ifdef PLIST

  if (newPlist)
    {
      playerlistnum ();
      playerlist ();
    }
#endif

#ifdef USE_TR_PORTS
  if (BoolVal[BOOL_USETRPORTS].value)
    printf ("Using Trace Route ports for UDP connection.\n");
#endif

  printf ("Done reading default values ...\n");
}

