
/*
 * playerlist.c
 *
 *   Fairly substantial re-write to do variable player lists: Sept 93 DRG
 */
#include "copyright.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "Wlib.h"
#include "defs.h"
#include "struct.h"
#include "data.h"

static char *my_classes[NUM_TYPES + 1] =
{
  "SC", "DD", "CA", "BB", "AS", "SB", "GA", "??"
};

#define STATUS 1			/* player died */
#define KILLS 2 			/* kills changed */
#define SHIP 4				/* refit to new ship type */
#define RACE 8				/* Changed the race -> sort list */
#define PNAME 16			/* New player */

#define FULL_UPDATE		STATUS | KILLS | SHIP | RACE | PNAME

static char header[BUFSIZ], *list;
int header_len;
int old_newPlist;

/*===========================================================================*/

#include "string_util.c"

static struct Plist
{
  int	refresh, idx;
  int	status, kills;
  short ship, race;
  char	text[BUFSIZ];
} plist [MAXPLAYER];

int 
playerlistnum ()
{
  int num = 0;
  char *ptr;

  D_plist[0] = '\0';

/* Must initialize the playerlist variables */
  for(num=0;num <= MAXPLAYER;num++) memset(plist[num].text, ' ', BUFSIZ);

  if (getdefault("playerlist") == NULL)
    {
      if (newPlist)
        strcpy(D_plist,"nTR N  K lrSd");
      else
        strcpy(D_plist,"nTRNKWLr O D d ");

    }
  else strcpy(D_plist,getdefault("playerlist"));

                                      
  old_newPlist = newPlist;


  ptr = D_plist;
  header[0] = '\0';
  while (ptr[0] != '\0')
    {
      switch (ptr[0])
	{
	case 'n':		/* Ship Number */
	  strcat (header, " No");
	  num += 3;
	  break;
	case 'T':		/* Ship Type */
	  strcat (header, " Ty");
	  num += 3;
	  break;
	case 'R':		/* Rank */
	  strcat (header, " Rank      ");
	  num += 11;
	  break;
	case 'N':		/* Name */
	  strcat (header, " Name            ");
	  num += 17;
	  break;
	case 'K':		/* Kills */
	  strcat (header, " Kills");
	  num += 6;
	  break;
	case 'l':		/* Login Name */
	  strcat (header, " Login           ");
	  num += 17;
	  break;
	case 'O':		/* Offense */
	  strcat (header, " Offse");
	  num += 6;
	  break;
	case 'W':		/* Wins */
	  strcat (header, "  Wins");
	  num += 6;
	  break;
	case 'D':		/* Defense */
	  strcat (header, " Defse");
	  num += 6;
	  break;
	case 'L':		/* Losses */
	  strcat (header, "  Loss");
	  num += 6;
	  break;
	case 'S':		/* Total Rating (stats) */
	  strcat (header, " Stats");
	  num += 6;
	  break;
	case 'r':		/* Ratio */
	  strcat (header, " Ratio");
	  num += 6;
	  break;
	case 'd':		/* Damage Inflicted (DI) */
	  strcat (header, "      DI");
	  num += 8;
	  break;
	case ' ':		/* White Space */
	  strcat (header, " ");
	  num += 1;
	  break;
#ifdef PLIST1
	case 'B':		/* Bombing */
	  strcat (header, " Bmbng");
	  num += 6;
	  break;
	case 'b':		/* Armies Bombed */
	  strcat (header, " Bmbed");
	  num += 6;
	  break;
	case 'P':		/* Planets */
	  strcat (header, " Plnts");
	  num += 6;
	  break;
	case 'p':		/* Planets Taken */
	  strcat (header, " Plnts");
	  num += 6;
	  break;
	case 'M':		/* Display, Host Machine */
	  strcat (header, " Host Machine    ");
	  num += 17;
	  break;
	case 'H':		/* Hours Played */
	  strcat (header, " Hours ");
	  num += 7;
	  break;
	case 'k':		/* Max Kills */
	  strcat (header, " Max K");
	  num += 6;
	  break;
#ifdef SBHOURS
	case 'V':		/* Kills per hour */
	  strcat (header, "   KPH");
	  num += 6;
	  break;
	case 'v':		/* Deaths per hour */
	  strcat (header, "   DPH");
	  num += 6;
	  break;
#endif
#endif
#ifdef PLIST2
	case 'w':		/* War staus */
	  strcat (header, " War Stat");
	  num += 9;
	  break;
	case 's':		/* Speed */
	  strcat (header, " Sp");
	  num += 3;
	  break;
#endif
	default:
	  fprintf (stderr, "%c is not an option for the playerlist\n", ptr[0]);
	  break;
	}
      ptr++;
    }
  header_len = num;
  return (num);
}

/*===========================================================================*/

playerlist ()
{
  int i;

  if (old_newPlist != newPlist)
    {
      playerlistnum ();
    }

  W_ClearWindow (playerw);

  W_WriteText (playerw, 0, 1, textColor, header, header_len, W_BoldFont);

  for (i = 0; i < MAXPLAYER; i++)
    {
      if (plist[i].status)
        plist[i].refresh++;
      updatePlayer[i]++;
    }

  playerlist2 ();
}

void    sort_playerlist(void)
{
  register int t, i, pos = 0;
  int     race[6];
  
  /* We're going to sort the list of races... */
  
  pos = 0;
  if (me->p_team != FED) {
    race[pos] = FED;
    pos++;
  }
  if (me->p_team != ROM) {
    race[pos] = ROM;
    pos++;
  }
  if (me->p_team != KLI) {
    race[pos] = KLI;
    pos++;
  }
  if (me->p_team != ORI) {
    race[pos] = ORI;
    pos++;
  }
  race[pos] = me->p_team;
  pos++;
  race[pos] = ALLTEAM;
  pos++;
  race[pos] = NOBODY;
  
/*  {FED, ROM, KLI, ORI, ALLTEAM, NOBODY} */

  pos = 0;  
  for (t = 0; t < (sizeof(race) / sizeof(int)); t++)

    for (i = 0; i < MAXPLAYER; i++)

      if ((plist[i].race == race[t]) && (plist[i].idx >= 0))
        {
          if (plist[i].idx != pos)
            {
              plist[i].idx = pos;
              plist[i].refresh++;
            }
          pos++;
        }
    
}


/*===========================================================================*/

plist_line (struct player *j,int pos)
{
  char 	*ptr;
  char 	tmp[30];
  int 	kills, losses, my_ticks;
  float	pRating, oRating, dRating, bRating, Ratings;
  double ratio, max_kills;
#ifdef SBHOURS
  float KillsPerHour, LossesPerHour;	/* Added 12/27/93 ATH */
#endif
  char *pl;
  int	update = 0;

  if (plist[pos].status |= j->p_status)
    {
      if (j->p_status == PFREE)
        {
          memset(plist[pos].text, ' ', BUFSIZ);
          plist[pos].refresh++;
          plist[pos].status = PFREE;
          plist[pos].race = NOBODY;
          return;
        }
      if ((plist[pos].status == PFREE) || (j->p_status == PALIVE))
        {
          if (plist[pos].idx <0) plist[pos].idx=pos;
          update = FULL_UPDATE;                  /* To be on the save side */
          if ((plist[pos].status == PFREE) &&
              (plist[plist[pos].idx].status))
            {
              plist[pos].status = j->p_status;
              sort_playerlist();                 /* fresh slot used */
            }
        }
      else
        update |= STATUS;
      plist[pos].status = j->p_status;
    }

  if (j->p_status == PFREE)
    {
      plist[pos].idx = -1;
      plist[pos].refresh = 0;
      return;
    }

  if (plist[pos].ship != j->p_ship.s_type)
    {
      update |= SHIP;
      plist[pos].ship = j->p_ship.s_type;
    }

  if (plist[pos].race != j->p_team)
    {
      update |= RACE;
      plist[pos].race = j->p_team;
      /* if (j->p_status == PALIVE) */
      sort_playerlist();
    }



  if ((plist[pos].kills != j->p_kills) || (update & KILLS))
    {
      update |= KILLS;
      plist[pos].kills = j->p_kills;

      if (j->p_ship.s_type == STARBASE)
        {
          kills = j->p_stats.st_sbkills;
          losses = j->p_stats.st_sblosses;
          max_kills = j->p_stats.st_sbmaxkills;
#ifdef SBHOURS
          if (SBhours)
	    {
	      my_ticks = j->p_stats.st_sbticks;
	    }
          else
	    {
	      my_ticks = j->p_stats.st_tticks;
	    }

          KillsPerHour = (float) (my_ticks == 0) ? 0.0 :
	    (float) kills *36000.0 / (float) my_ticks;
          LossesPerHour = (float) (my_ticks == 0) ? 0.0 :
	    (float) losses *36000.0 / (float) my_ticks;
#else
          my_ticks = j->p_stats.st_sbticks;
#endif
        }
  else
        {
          kills = j->p_stats.st_kills + j->p_stats.st_tkills;
          losses = j->p_stats.st_losses + j->p_stats.st_tlosses;
          max_kills = j->p_stats.st_maxkills;
          my_ticks = j->p_stats.st_tticks;
#ifdef SBHOURS
          KillsPerHour = (float) (my_ticks == 0) ? 0.0 :
	    (float) j->p_stats.st_tkills * 36000.0 / (float) my_ticks;
          LossesPerHour = (float) (my_ticks == 0) ? 0.0 :
	    (float) j->p_stats.st_tlosses * 36000.0 / (float) my_ticks;
#endif
        }


      if (losses == 0)
        {
          ratio = (double) kills;
        }
      else
        {
          ratio = (double) kills / (double) losses;
        }

      if (!j->p_stats.st_tticks)
        {
          oRating = pRating = bRating = Ratings = 0.0;
          dRating = defenseRating (j);
        }
      else
        {
          oRating = offenseRating (j);
          pRating = planetRating (j);
          bRating = bombingRating (j);
          Ratings = oRating + pRating + bRating;
          dRating = defenseRating (j);
#ifdef SHOURS
          if ((j->p_ship.s_type == STARBASE) && (SBhours))
	    {			/* If SB, show KPH for offense etc. */
	      oRating = KillsPerHour;
	      dRating = LossesPerHour;
	    }
#endif
        }
    }

  if (!update)
    return;

  plist[pos].refresh++;
  ptr = D_plist;
  pl = (char *) &(plist[pos].text);
  while (ptr[0] != '\0')
    {
      switch (ptr[0])
	{
	case 'n':		/* Ship Number */
          if (update & (STATUS | RACE))
            {
	      if (j->p_status != PALIVE)
	        {
	          *pl++ = ' ';
	          *pl++ = ' ';
	          *pl++ = shipnos[j->p_no];
	        }
	      else
		{
		  *pl++ = ' ';
		  *pl++ = teamlet[j->p_team];
		  *pl++ = shipnos[j->p_no];
		}
	    }
	  else
	    pl += 3;
	  break;
	case 'T':				 /* Ship Type */
	  if (update & (STATUS | SHIP))
	    {
	      if (j->p_status != PALIVE)
		{
		  *pl++ = ' ';
		  *pl++ = ' ';
		  *pl++ = ' ';
		}
	      else
		{
		  *pl++ = ' ';
		  *pl++ = my_classes[j->p_ship.s_type][0];
		  *pl++ = my_classes[j->p_ship.s_type][1];
		}
	    }
	  else
	    pl += 3;
	  break;
	case 'R':				 /* Rank */
	  if (update & KILLS)
	    {
	      *pl = ' ';
	      format(tmp, ranks[j->p_stats.st_rank].name, 10, 0);
	      strncpy(pl + 1, tmp, 10);
	    }
	  pl += 11;
	  break;
	case 'N':				 /* Name */
	  if (update & PNAME)
	    {
	      *pl = ' ';
	      format(tmp, j->p_name, 16, 0);
	      strncpy(pl + 1, tmp, 16);
	    }
	  pl += 17;
	  break;
	case 'K':				 /* Kills */
	  if (update & KILLS)
	    {
	      *pl = ' ';
	      ftoa(j->p_kills, tmp, 0, 2, 2);
	      strncpy(pl + 1, tmp, 5);
	    }
	  pl += 6;
	  break;
	case 'l':				 /* Login Name */
	  if (update & PNAME)
	    {
	      *pl = ' ';
	      format(tmp, j->p_login, 16, 0);
	      strncpy(pl + 1, tmp, 16);
	    }
	  pl += 17;
	  break;
	case 'O':				 /* Offense */
	  if (update & KILLS)
	    {
	      *pl = ' ';
	      ftoa(oRating, tmp, 0, 2, 2);
	      strncpy(pl + 1, tmp, 5);
	    }
	  pl += 6;
	  break;
	case 'W':				 /* Wins */
	  if (update & KILLS)
	    {
	      *pl = ' ';
	      dectoa(kills, tmp, 0, 5);
	      strncpy(pl + 1, tmp, 5);
	    }
	  pl += 6;
	  break;
	case 'D':				 /* Defense */
	  if (update & KILLS)
	    {
	      *pl = ' ';
	      ftoa(dRating, tmp, 0, 2, 2);
	      strncpy(pl + 1, tmp, 5);
	    }
	  pl += 6;
	  break;
	case 'L':				 /* Losses */
	  if (update & KILLS)
	    {
	      *pl = ' ';
	      dectoa(losses,tmp, 0, 5);
	      strncpy(pl + 1, tmp, 5);
	    }
	  pl += 6;
	  break;
	case 'S':				 /* Total Rating (stats) */
	  if (update & KILLS)
	    {
	      *pl = ' ';
	      ftoa(Ratings, tmp, 0, 2, 2);
	      strncpy(pl + 1, tmp, 5);
	    }
	  pl += 6;
	  break;
	case 'r':				 /* Ratio */
	  if (update & KILLS)
	    {
	      *pl = ' ';
	      ftoa(ratio, tmp, 0, 2, 2);
	      strncpy(pl + 1, tmp, 5);
	    }
	  pl += 6;
	  break;
	case 'd':				 /* Damage Inflicted (DI) */
	  if (update & KILLS)
	    {
	      *pl = ' ';
	      ftoa(Ratings * (j->p_stats.st_tticks / 36000.0), tmp, 0, 4, 2);
	      strncpy(pl + 1, tmp, 7);
	    }
	  pl += 8;
	  break;
	case ' ':				 /* White Space */
	  *pl++ = ' ';
	  break;

#ifdef PLIST1
	case 'B':				 /* Bombing */
	  if (update & KILLS)
	    {
	      *pl = ' ';
	      ftoa(bRating, tmp, 0, 2, 2);
	      strncpy(pl + 1, tmp, 5);
	    }
	  pl += 6;
	  break;
	case 'b':				 /* Armies Bombed */
	  if (update & KILLS)
	    {
	      *pl = ' ';
	      dectoa(j->p_stats.st_tarmsbomb + j->p_stats.st_armsbomb, tmp, 0, 5);
	      strncpy(pl + 1, tmp, 5);
	    }
	  pl += 6;
	  break;
	case 'P':				 /* Planets */
	  if (update & KILLS)
	    {
	      *pl = ' ';
	      ftoa(pRating, tmp, 0, 2, 2);
	      strncpy(pl + 1, tmp, 5);
	    }
	  pl += 6;
	  break;
	case 'p':				 /* Planets Taken */
	  if (update & KILLS)
	    {
	      *pl = ' ';
	      dectoa(j->p_stats.st_tplanets + j->p_stats.st_planets, tmp, 0, 5);
	      strncpy(pl + 1, tmp, 5);
	    }
	  pl += 6;
	  break;
	case 'M':				 /* Display, Host Machine */
	  if (update & PNAME)
	    {
	      *pl = ' ';
	      format(tmp, j->p_monitor, 16, 0);
	      strncpy(pl + 1, tmp, 16);
	    }
	  pl += 17;
	  break;
	case 'H':				 /* Hours Played */
	  if (update & KILLS)
	    {
	      *pl = ' ';
	      ftoa(my_ticks / 36000.0, tmp, 0, 3, 2);
	      strncpy(pl + 1, tmp, 6);
	    }
	  pl += 7;
	  break;
	case 'k':				 /* Max Kills  */
	  if (update & KILLS)
	    {
	      *pl = ' ';
	      ftoa(max_kills, tmp, 0, 2, 2);
	      strncpy(pl + 1, tmp, 5);
	    }
	  pl += 6;
	  break;
	case 'V':				 /* Kills Per Hour  */
	  if (update & KILLS)
	    {
	      *pl = ' ';
	      ftoa(KillsPerHour, tmp, 0, 3, 1);
	      strncpy(pl + 1, tmp, 5);
	    }
	  pl += 6;
	  break;
	case 'v':				 /* Deaths Per Hour  */
	  if (update & KILLS)
	    {
	      *pl = ' ';
	      ftoa(LossesPerHour, tmp, 0, 3, 1);
	      strncpy(pl + 1, tmp, 5);
	    }
	  pl += 6;
	  break;
#endif

#ifdef PLIST2
	case 'w':				 /* War status */
	  if (j->p_swar & me->p_team)
	    strncpy(pl, " WAR     ", 9);
	  else if (j->p_hostile & me->p_team)
	    strncpy(pl, " HOSTILE ", 9);
	  else
	    strncpy(pl, " PEACEFUL", 9);
	  pl += 9;
	  break;
	case 's':				 /* Speed */
	  *pl = ' ';
	  dectoa(j->p_speed, tmp, 0, 2);
	  strncpy(pl + 1, tmp, 2);
	  pl += 3;
	  break;
#endif

	default:
	  break;
	}
      ptr++;
    }


}

/*===========================================================================*/

playerlist2(void)
{
  register int i;
  register struct player *j;

  if (!W_IsMapped(playerw))
    return;

  for (i = 0, j = &players[i]; i < MAXPLAYER; i++, j++)
    {
      if (!updatePlayer[i])
        continue;

      updatePlayer[i] = 0;
      plist_line(j, i);

    }

  for (i = 0, j = &players[i]; i < MAXPLAYER; i++, j++)
    if (plist[i].refresh)
      {
        if (sortPlayers)
          W_WriteText(playerw, 0, plist[i].idx + 2, playerColor(j),
                      plist[i].text, header_len, shipFont(j));
        else
          W_WriteText(playerw, 0, i + 2, playerColor(j),
                      plist[i].text, header_len, shipFont(j));
        plist[i].refresh = 0;
      }
}
