/*
 * winmain.c
 * Main file for the MS-Windows port of Netrek. Contains the "main" function,
 * as well as Windows implemenations of major Unix functions. Handles differences
 * between NT and Win32s, specifically the lack of consoles on Win32s.
 *
 * Copyright (C) 1993-1994 Jonathan Shekter
 */

#define NO_BOOLEAN

#include <windows.h>
#include <windowsx.h>
#include <winsock.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pwd.h>
#include "Wlib.h"
#include "defs.h"
#include "struct.h"
#include "data.h"

#ifdef __BORLANDC__
   #pragma warn -par  //so we don't get a zillion parameter never used messages
#endif

//Global data
HINSTANCE MyInstance;
char ExeDir[100];
char HomeDir[100];
W_Window console;
int PrintfSafe = 0;     //Turns off printf during critical operations
BOOL fWin32s;

void Wprintf(char *format,...);
void Wfprintfstderr(char *format, ...);
void WinMainCleanup();

char *GetExeDir()
   {
   return ExeDir;
   }

//Need _iob array on Win32s for std handles (missing on Win32s CRTDLL.DLL)
FILE realiob[3];
FILE *_iob = realiob;



//Our main. Does some init before calling the actual Netrek code, which
//now has main2() instead of main().
int main(int argc, char **argv)
   {
   char *c;
   WSADATA data;                                   //Using sockets - perform WinSock startup

   MyInstance = GetModuleHandle(NULL);

   fWin32s = (getenv("FORCECONSOLE") ? 1 : GetVersion() & 0x80000000);

   //Get home path (screwny NT environ vars), and set HOME var
   if (c=getenv("HOMEDRIVE"))
      {
      strcpy(HomeDir, "HOME=");
      strcat(HomeDir, c);
      if (c=getenv("HOMEPATH"))
         strcat(HomeDir, c);
      if (HomeDir[strlen(HomeDir)-1] != '\\')
      	strcat(HomeDir, "\\");
      putenv(HomeDir);
      }

   //Get the executable file directory
   c = ExeDir + GetModuleFileName(MyInstance, ExeDir, sizeof(ExeDir)) -1;
   while (*c != '\\')   //remove filename
      *(c--) = 0;
   *c = 0;  //remove last backslash


   //Kick winsock into gear
   if (WSAStartup(MAKEWORD(1,1), &data) != 0)
      {
      Wfprintfstderr("Could not find WINSOCK.DLL / WINSOCK initializtion failed");
      return -1;
      }
   if ( data.wVersion<MAKEWORD(1,1) )
      {
      Wfprintfstderr("Bad WINSOCK.DLL version (need at least v1.1).");
      WSACleanup();
      return -1;
      }

   //Create the "Console" window if running on Win32s
   W_Initialize();
   if (fWin32s)
      {
      console = W_MakeScrollingWindow("Console", 0, 0, 80, 10, NULL, 2);
      PrintfSafe = 1;                                 //Now OK to printf
      }

   atexit(WinMainCleanup);

   main2(argc, argv);                                 //Call BRM main!

   return 0;
   }


void WinMainCleanup()
   {
   if (fWin32s)
      {
      //If we're using our own special console window,
      //pause before closing so the user can read
      Wprintf("Netrek/Win32 terminating...");
      W_DestroyWindow(console);
      PrintfSafe=0;        //Console is dead
      }

   WSACleanup();        //Shut down our sockets
   }


//***************************************************************************//
// Various UNIX functions that will enable the code to compile as-is without //
// changes - only this file added.                                           //
//***************************************************************************//

//Get process ID
int getpid()
   {
   return (int) GetCurrentProcessId();
   }


//Get user ID - fugde!
char *getuid()
   {
   return (char *)1;
   }

//getpwuid() function - does it for real under NT. Just copies the given string...
struct passwd *getpwuid(char *getuidstuff)
   {
   static struct passwd pw;
   static char name[100];
   int i= sizeof(name);

   if (WNetGetUser(NULL, name, &i) != NO_ERROR )
      strcpy(name, "Bozo"); //Use "bozo" if function fails (on Windows 3.1?)

   pw.pw_name = name;
   return &pw;
   }

//Sleep...
void sleep(int seconds)
   {
   Sleep(seconds * 1000);
   }


//******************************** MISC STUFF ***************************888//

//A useful function to replace printf().
void Wprintf(char *format,...)
   {
   char buf[300],*vargs;
   vargs=(char *) &format + sizeof(format);
   vsprintf(buf,format,vargs);
   MessageBox(NULL,buf,"Netrek/Win32",MB_OK);
   }

//Another function, this one to replace fprintf(stderr,...)
void Wfprintfstderr(char *format, ...)
   {
   char buf[300],*vargs;
   vargs=(char *) &format + sizeof(format);
   vsprintf(buf,format,vargs);
   MessageBeep(MB_ICONEXCLAMATION);
   MessageBox(NULL,buf,"Netrek/Win32: Error",MB_OK|MB_ICONEXCLAMATION);
   }


/* - The following fns not needed in Win32, as we actually have a real, live console of our own
   - But, under Win32s we don't... so... arrgh */


//Yet another function, this one actually IS printf() - overrides the default
//definition, sends it to the console window -
int printf(const char *format,...)
   {
   char buf[300],*vargs;
   int len, junk;

   vargs=(char *) &format + sizeof(format);
   len = vsprintf(buf,format,vargs);

   if (fWin32s)
     {
     if (!PrintfSafe)
        {
   	MessageBox(NULL, buf, "This is a printf", MB_OK);
   	return 0; //Flag that helps us avoid infinite recursions and other nasties
   	}
      PrintfSafe = 0;
      if (!W_IsMapped(console))
         W_MapWindow(console);
      W_WriteText(console, 0, 0, W_White, buf, len, W_RegularFont);
      W_FlushScrollingWindow(console);
      PrintfSafe = 1;
      }
   else
     WriteFile(GetStdHandle(STD_OUTPUT_HANDLE), buf, len, &junk, NULL);

   return len;
   }

//Arrghh our version of fwrite...
int myfwrite(char *buf, int count1, FILE *file)
   {
   register int count = count1;
   while (count--)
      {
      if (fputc(*buf, file) == EOF)
         return ((count1 - count)-1);
      buf++;
      }
   return count1;
   }


//Redriect fprintf() to stdout and stderr to our console window under Win32s
int fprintf(FILE *handle, const char *format,...)
   {
   char buf[300],*vargs;
   int len, junk;

   vargs=(char *) &format + sizeof(format);
   len = vsprintf(buf,format,vargs);

   if (handle == stdout || handle == stderr)
         {
         if (fWin32s)
            {
            if (!PrintfSafe)
         	{
		MessageBox(NULL, buf, "This is an fprintf to stderr/stdout", MB_OK);
         	return 0;
         	}
            PrintfSafe = 0;
            if (!W_IsMapped(console))
               W_MapWindow(console);
            W_WriteText(console, 0, 0, W_White, buf, len, W_RegularFont);
            W_FlushScrollingWindow(console);
            PrintfSafe = 1;
            }
         else
            WriteFile(GetStdHandle(STD_OUTPUT_HANDLE), buf, len, &junk, NULL);
         }
   else
      return myfwrite(buf, len, handle);

   return len;
   }


//More nastyness: this time for fwrite. I hate Win32s. No consoles!
int fwrite(char *buf, int blksz, int count, FILE *handle)
   {
   int junk;

   if (handle == stdout || handle == stderr)
         {
         if (fWin32s)
            {
            if (!PrintfSafe)
         	{
		MessageBox(NULL, buf, "This is an fwrite to stderr/stdout", MB_OK);
         	return 0;
         	}
            PrintfSafe = 0;
            if (!W_IsMapped(console))
               W_MapWindow(console);
            W_WriteText(console, 0, 0, W_White, buf, blksz*count, W_RegularFont);
            W_FlushScrollingWindow(console);
            PrintfSafe = 1;
            }
         else
            WriteFile(GetStdHandle(STD_OUTPUT_HANDLE), buf, blksz*count, &junk, NULL);
         }
   else
      return myfwrite(buf, count*blksz, handle);

   return blksz*count;
   }


//A more usefule  perror(), reports last winsock error as well
void perror(const char *str)
   {
   printf("%s: errno = %d, WSALast = %d\n", str, errno, WSAGetLastError());
   }
